/*
 * ASCD : copyright (c) Aley Keprt 1998-2001
 *
 * File joy.cpp - joystick emulation code
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "allegro.h"

#include "joy.h"
#include "samio.h"
#include "resource.h"
#include "util.h"
#include "kbd.h"


JState arrows;


static inline bool operator!=(const JState &a,const JState &b) {
  return a.left!=b.left || a.right!=b.right ||
    a.up!=b.up || a.down!=b.down || a.fire!=b.fire;
}


void JoyInit() {
//
//initialize joystick emulation
//
  //muze se volat i opakovane
  if(install_joystick(JOY_TYPE_AUTODETECT) || joy_type==JOY_TYPE_NONE) return;

  OnQuit(remove_joystick,"remove_joystick()");

  simcoupe_mesg(SIMCOUPE_INFO,"found %i joysticks",num_joysticks);
  simcoupe_mesg(SIMCOUPE_INFO,"joystick: axes=%i, buttons=%i",
                joy[0].stick[0].num_axis,joy[0].num_buttons);
}


int JoyGetNum(void) {
//
//return number of joysticks
//
  return num_joysticks;
}


static void readjoy(JState &j,int i) {
//
//read joystick state
//
  const JOYSTICK_INFO &joy=::joy[i];
  const JOYSTICK_AXIS_INFO *axis=joy.stick[0].axis;

  j.left =axis[0].d1;
  j.right=axis[0].d2;
  j.up   =axis[1].d1;
  j.down =axis[1].d2;
  j.fire=0;
  for(int i=joy.num_buttons-1;i>=0;i--) j.fire|=joy.button[i].b;
}

/*
static void getchanges(JState &j, const JState last) {
//
//zjisti zmeny oproti predchozimu stavu joysticku
//
  if(last.left && !j.left)
}
*/

void JoyUpdate() {
//
//poll joysticks etc.
//
  int polled=0;
  JState j;
  static JState lastj1,lastj2,lastcj;

#define test(model,set)                          \
  if(model>-1 && model<=num_joysticks) {         \
    if(model==0) {                               \
      /* arrows */                               \
      j=arrows;                                  \
      /* turn off cntrl & symbol when fire*/     \
  /*  if(j.fire)                             */  \
  /*  if(cfg.zxmode==EM_Sam) keyports[8]|=1; */  \
  /*  else keyports[7]|=2;                   */  \
    } else {                                     \
      /* joystick */                             \
      if(!polled) {poll_joystick();polled=1;}    \
      readjoy(j,model-1);                        \
    }                                            \
    set;                                         \
  }

  test(cfg.kempston,
    kjport=0xe0+j.right+j.left*2+j.down*4+j.up*8+j.fire*0x10;
    );

//makro update slouzi k vnuceni stavu klaves pouze pri zmene stavu joysticku
#define update(last,cmd) if(j!=last) {last=j; cmd;}

  test(cfg.sinclair1,update(lastj1,keyports[4]=(keyports[4]|0x1f)-j.left*0x10-j.right*8-j.down*4-j.up*2-j.fire));

  test(cfg.sinclair2,update(lastj2,keyports[3]=(keyports[3]|0x1f)-j.left-j.right*2-j.down*4-j.up*8-j.fire*0x10));

  //no cursor & fuller joysticks in Sam mode
  if(cfg.zxmode!=EM_Sam) {

    test(cfg.cursor,update(lastcj,
         keyports[4]=(keyports[4]|0x1d)-j.right*4-j.down*0x10-j.up*8-j.fire;
         keyports[3]=(keyports[3]|0x10)-j.left*0x10));

    test(cfg.fuller,fuller=0xff-j.fire*0x80-j.left*4-j.right*8-j.down*2-j.up);
  }
}

