/* 
 * ASCD : copyright (c) Aley Keprt 1998-2001
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File z80.h - defines the processor and its registers.
 *              Also declares some common functions.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef __Z80_H
#define __Z80_H


#define TIMER_SPEED 50  //sync timer speed (usually 50Hz)


void reinit_sync_timer(void);


/* Sam timing constants */
#define TSTATES_PER_LINE	384   /* tstates per generated line - Sam mode */
#define TSTATES_PER_LINE_ZX     224   /* tstates per generated line - ZXS mode */
#define TSTATES_PER_LINE_128    228


enum {
  Z80_none,
  Z80_quit,
  Z80_nmi,
  Z80_reset,
  Z80_menu,
};


/* Constants used to bit-mask the F register */
#define F_CARRY		0x01
#define F_NADD		0x02	/* For BCD. */
#define F_PARITY	0x04
#define F_OVERFLOW	0x04
#define F_HCARRY	0x10	/* For BCD. */
#define F_ZERO		0x40
#define F_NEG		0x80


typedef struct {
  uns16 pc, ix, iy, sp;
  uns8 a, b, c, f, d, e, h, l;
  uns8 r, a1, f1, b1, c1, d1, e1, h1, l1, i, iff1, iff2, im;
} Z80_Regs;


extern int InterruptType;


void init_ports(void);
void emulator_main_loop(void);
void set_registers(Z80_Regs *r);


#endif
